//-*-c++-*-
/***************************************************************************
 *   Copyright (C) 2003 by Fred Schaettgen                                 *
 *   kdebluetooth@schaettgen.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "atcommander.h"
#include <libkbluetooth/rfcommsocket.h>
#include <qdatetime.h>
#include <qapplication.h>
#include <kdebug.h>

AtCommander::AtCommander(QObject *owner, const char* name,
    KBluetooth::RfcommSocket *socket) :
    QObject(owner, name)
{
    this->socket = socket;
    connect(this->socket, SIGNAL(readyRead()),
        this, SLOT(slotSocketReadyRead()));
}

bool AtCommander::sendAtCommand(QString cmd, double timeout,
    bool waitForResult)
{
    if (!socket) return false;

    QString s = QString("AT") + cmd + "\r";
    QCString buf(s.ascii());
    socket->writeBlock((const char*)buf, buf.length());

    emit lineSent(QString("AT")+cmd);

    if (waitForResult) {
        // Wait for the result
        QDateTime startTime = QDateTime::currentDateTime();
        this->waitingForResult = true;
        this->lastResultOK = false;
        do {
            // Busy wait for result data
            qApp->processEvents(500);
            //kdDebug() << "waiting for result" << endl;
        } while ((waitingForResult == true) &&
            (startTime.secsTo(QDateTime::currentDateTime()) < timeout));
        kdDebug() << "done " << waitingForResult << " "
            << lastResultOK << endl;
        return lastResultOK && (waitingForResult == false);
    }
    else return true;
}


void AtCommander::slotSocketReadyRead()
{
    if (!socket) return;

    kdDebug() << "slotSocketReadyRead" << endl;
    // Receive as all pending data and
    // append it to the recvBuf
    const int len = socket->size();
    if (len > 0) {
        char *buf = new char[len];
        socket->readBlock(buf, len);
        QCString cs(buf, len+1);
        delete []buf;
        this->recvBuf += QString(cs);
    }

    // Consume complete result lines from
    // the recvBuf
    int pos;
    while ((pos = recvBuf.find("\r\n",1)) >= 0) {
        QString line = recvBuf.left(pos+2);
        recvBuf = recvBuf.remove(0, pos+2);
        line = line.replace("\n", "").replace("\r", "");
        emit lineReceived(line);

        if (line.upper().startsWith("OK")) {
            waitingForResult = false;
            lastResultOK = true;
        }
        else if (line.upper().startsWith("ERROR")) {
            waitingForResult = false;
            lastResultOK = false;
        }
        else {
            emit unsolicitedResult(line);
        }
    }
}


AtCommander::~AtCommander()
{
}

#include "atcommander.moc"
