//-*-c++-*-
/***************************************************************************
 *   Copyright (C) 2003 by Fred Schaettgen                                 *
 *   kdebluetooth@schaettgen.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "configuredialog.h"
#include "configrecording.h"
#include <qvbox.h>
#include <qbuttongroup.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kurlrequester.h>
#include <kfiledialog.h>
#include <klocale.h>

ConfigureDialog::ConfigureDialog(QWidget* parent) :
    KDialogBase(KDialogBase::Tabbed, "config",
        KDialogBase::Ok|KDialogBase::Cancel|KDialogBase::Apply,
        KDialogBase::Ok, parent)
{
    QVBox* recordingBox = addVBoxPage(i18n("Recording"));
    recordingTab = new ConfigRecordingWidget(recordingBox, "recordingTab");

    connect(this, SIGNAL(applyClicked()), this, SLOT(saveConfig()));
    connect(this, SIGNAL(okClicked()), this, SLOT(saveConfig()));

    readConfig();
}


void ConfigureDialog::readConfig()
{
    KConfig* config = KGlobal::config();
    config->setGroup("recording");
    recordingTab->targetPath->setURL(config->readEntry("targetPath", "/tmp"));
    recordingTab->recordingGroup->setButton(config->readNumEntry("recordingMode", 0));
    recordingTab->targetPath->setMode(KFile::Directory |
        KFile::ExistingOnly | KFile::LocalOnly);
}

void ConfigureDialog::saveConfig()
{
    KConfig* config = KGlobal::config();
    config->setGroup("recording");
    config->writeEntry("targetPath", recordingTab->targetPath->url());
#if (QT_VERSION >= 0x030200)
    config->writeEntry("recordingMode", recordingTab->recordingGroup->selectedId());
#else
    config->writeEntry("recordingMode", recordingTab->recordingGroup->id(recordingTab->recordingGroup->selected()));
#endif
}


ConfigureDialog::~ConfigureDialog()
{
}


#include "configuredialog.moc"
