//-*-c++-*-
/***************************************************************************
 *   Copyright (C) 2003 by Fred Schaettgen                                 *
 *   kdebluetooth@schaettgen.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "mainwindow.h"
#include <kuniqueapplication.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kurl.h>
#include <libkbluetooth/hcidefault.h>
#include <stdlib.h>

static const char description[] =
    I18N_NOOP("Bluetooth Handsfree");

static const char version[] = "0.1";

static KCmdLineOptions options[] =
    {
        {"phonenumber <number>", I18N_NOOP("Phone number to dial."), 0},
        {"dial", I18N_NOOP("Immediately dial the number."), 0},
        {"rfcommconnection <sockfd>", I18N_NOOP("Used by kbluetoothd."), 0},
        {"peeraddr <addr>", I18N_NOOP("Used by kbluetoothd."), 0},
        {"peername <name>", I18N_NOOP("Used by kbluetoothd."), 0},
        {"+[sdpurl]", I18N_NOOP("Used by kio_sdp"), 0 },
        KCmdLineLastOption
    };

using namespace KBluetooth;

class HandsfreeApplication : public KUniqueApplication
{
public:
    virtual int newInstance()
    {
        KGlobal::locale()->insertCatalogue("libkbluetooth");
        KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
        bool instantDial = args->isSet("dial");
        mainWindow->setNumber(args->getOption("phonenumber"));

        if (args->isSet("rfcommconnection") && args->isSet("peeraddr") &&
            args->isSet("peername")) {
            int socket = QString(args->getOption("rfcommconnection"))
                .toInt();
            QString name = QString(args->getOption("peername"));
            QString addr = QString(args->getOption("peeraddr"));

            mainWindow->openConnection(socket, addr, name);
            if (instantDial == true) {
                mainWindow->dialOnNextConnection();
            }
        }
        else if (args->isSet("phonenumber")) {
            mainWindow->openConnection();
            if (instantDial == true) {
                mainWindow->dialOnNextConnection();
            }
        }
        else if (args->count() == 0) {
            mainWindow->openConnection();
        }
        else if (args->count() == 1) {
            KURL url = args->arg(0);
            QString host = url.host();
            // Get the selected sdp service record handle
            QString uuid = url.queryItem("recordhandle");
            // Get the rfcomm channel
            QString channel = url.queryItem("rfcommchannel");

            mainWindow->openConnection(DeviceAddress(host), channel.toInt());
        }

        return 0;
    }

    /*static bool sendSocket(int fd) {
        struct sockaddr_un unix_socket_name = {0};
        int unix_socket_fd;

        // Open a well-known socket in /tmp
        unix_socket_name.sun_family = AF_UNIX;
        strcpy(unix_socket_name.sun_path, "/tmp/kbthandsfree.connection_transfer");
        unix_socket_fd = socket(PF_UNIX, SOCK_DGRAM, 0);
        if (unix_socket_fd == -1) return false;
        // doesn't do anything at the moment:
        // connect(unix_socket_fd, (sockaddr*)&unix_socket_name);

        struct msghdr msg;
        char ccmsg[CMSG_SPACE(sizeof(fd))];
        struct cmsghdr *cmsg;
        struct iovec vec;  // stupidity: must send/receive at least one byte
        char *str = "x";
        bool rv;

        msg.msg_name = (struct sockaddr*)&unix_socket_name;
        msg.msg_namelen = sizeof(unix_socket_name);

        vec.iov_base = str;
        vec.iov_len = 1;
        msg.msg_iov = &vec;
        msg.msg_iovlen = 1;

        msg.msg_control = ccmsg;
        msg.msg_controllen = sizeof(ccmsg);
        cmsg = CMSG_FIRSTHDR(&msg);
        cmsg->cmsg_level = SOL_SOCKET;
        cmsg->cmsg_type = SCM_RIGHTS;
        cmsg->cmsg_len = CMSG_LEN(sizeof(fd));
        *(int*)CMSG_DATA(cmsg) = fd;
        msg.msg_controllen = cmsg->cmsg_len;

        msg.msg_flags = 0;

        rv = (sendmsg(unix_socket_fd, &msg, 0) != -1);
        close(fd);
        close(unix_socket_fd);
        return true;
    }*/

    MainWindow* mainWindow;
};


int main(int argc, char **argv)
{
    KAboutData about("kbthandsfree",
        I18N_NOOP("Bluetooth Handsfree"),
        version, description,
        KAboutData::License_GPL,
        "(C) 2003 Fred Schaettgen", 0, 0,
        "kdebluetooth@0xF.de");
    KCmdLineArgs::init(argc, argv, &about);
    KCmdLineArgs::addCmdLineOptions( options );
    KUniqueApplication::addCmdLineOptions();
    KBluetooth::HciDefault::addCmdLineOptions();

    if (!KUniqueApplication::start()) {
        //KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
        fprintf(stderr, "kbthandsfree is already running!\n");
        exit(0);
    }
    HandsfreeApplication a;
    MainWindow *mainWindow = new MainWindow();
    a.mainWindow = mainWindow;
    a.setMainWidget(mainWindow);
    mainWindow->show();
    a.exec();

    return 0;
}

