//-*-c++-*-
/***************************************************************************
 *   Copyright (C) 2003 by Fred Schaettgen                                 *
 *   kdebluetooth@schaettgen.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "vorbisencoder.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>


VorbisEncoder::VorbisEncoder(QObject* parent, QString filename) :
    QObject(parent),
    outfile(filename)
{
    outfile.open(IO_WriteOnly);
    bInit = false;
}

QString VorbisEncoder::filename()
{
    return outfile.name();
}

VorbisEncoder::~VorbisEncoder()
{
    vorbis_analysis_wrote(&vd,0);

    ogg_stream_clear(&os);
    vorbis_block_clear(&vb);
    vorbis_dsp_clear(&vd);
    vorbis_comment_clear(&vc);
    vorbis_info_clear(&vi);
}


bool VorbisEncoder::init()
{
    vorbis_info_init(&vi);
    int ret=vorbis_encode_init_vbr(&vi,2,8000,.3);
    if (ret != 0) return false;

    /* add tags */
    vorbis_comment_init(&vc);
    QValueList<QString> keyList = tagMap.keys();
    for (int n=keyList.count()-1; n>=0; n--) {
        QString key = keyList[n];
        QString value = tagMap[key];
        vorbis_comment_add_tag(&vc, key.utf8().data(),
            value.utf8().data());
    }

    /* set up the analysis state and auxiliary encoding storage */
    vorbis_analysis_init(&vd,&vi);
    vorbis_block_init(&vd,&vb);

    /* set up our packet->stream encoder */
    /* pick a random serial number; that way we can more likely build
        chained streams just by concatenation */
    srand(time(NULL));
    ogg_stream_init(&os, rand());

    ogg_packet header;
    ogg_packet header_comm;
    ogg_packet header_code;

    vorbis_analysis_headerout(&vd, &vc, &header, &header_comm, &header_code);
    ogg_stream_packetin(&os, &header); /* automatically placed in its own
                                         page */
    ogg_stream_packetin(&os, &header_comm);
    ogg_stream_packetin(&os, &header_code);

    /* This ensures the actual
    * audio data will start on a new page, as per spec
    */
    while (true) {
            int result = ogg_stream_flush(&os,&og);
            if (result == 0) break;
            outfile.writeBlock((char*)og.header, og.header_len);
            outfile.writeBlock((char*)og.body, og.body_len);
    }
    bInit = true;
    return true;
}


void VorbisEncoder::writeBlock(Q_INT16* dataLeft, Q_INT16* dataRight, int datalen)
{
    //int READ = datasize/4;

    if (bInit == false) return;

    //long bytes=fread(readbuffer,1,READ*4,stdin); /* stereo hardwired here */

    /* data to encode */

    /* expose the buffer to submit data */
    float **buffer=vorbis_analysis_buffer(&vd,datalen);

    /* uninterleave samples */
    for(int i=0;i<datalen;i++){
        buffer[0][i] = (dataLeft[i]/32768.f)+0.5;
        //buffer[0][i*2+0] = buffer[0][i*2+1] = (dataLeft[i]/32768.f)+0.5;
        buffer[1][i] = (dataRight[i]/32768.f)+0.5;
        //buffer[1][i*2+0] = buffer[1][i*2+1] = (dataRight[i]/32768.f)+0.5;
    }

    /* tell the library how much we actually submitted */
    vorbis_analysis_wrote(&vd,datalen);

    /* vorbis does some data preanalysis, then divvies up blocks for
       more involved (potentially parallel) processing.  Get a single
       block for encoding now */
    while(vorbis_analysis_blockout(&vd,&vb)==1) {

        /* analysis, assume we want to use bitrate management */
        vorbis_analysis(&vb,NULL);
        vorbis_bitrate_addblock(&vb);

        while(vorbis_bitrate_flushpacket(&vd,&op)){
        /* weld the packet into the bitstream */
            ogg_stream_packetin(&os,&op);

            /* write out pages (if any) */
            while(true) {
                int result=ogg_stream_pageout(&os,&og);
                if(result==0)break;
                outfile.writeBlock((char*)og.header,og.header_len);
                outfile.writeBlock((char*)og.body,og.body_len);

                /* this could be set above, but for illustrative purposes, I do
                    it here (to show that vorbis does know where the stream ends) */

                if(ogg_page_eos(&og)) break;
            }
        }
    }
}

void VorbisEncoder::setTag(QString key, QString value)
{
    tagMap[key] = value;
}

QString VorbisEncoder::tag(QString key)
{
    if (tagMap.find(key) != tagMap.end()) {
        return tagMap[key];
    }
    else {
        return QString::null;
    }
}

#include "vorbisencoder.moc"
