/***************************************************************************
                          irmcsynckonnector.cpp  -  description
                             -------------------
    begin                : Sat Feb 14 2003
    copyright            : (C) 2003 by Simone Gotti
	
	from the threaded konnector of Mathias Froehlich <Mathias.Froehlich@web.de>
	
    email                : simone.gotti@email.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <kdebug.h>
#include <kgenericfactory.h>
#include <kconfig.h>
#include <konnectorinfo.h>

#include "clientmanager.h"

#include "obexclient.h"
#include "irmcsynckonnector.h"
#include "irmcsyncconfig.h"


using namespace KSync;

extern "C"
{
	void *init_libirmcsynckonnector() {
		return new KRES::PluginFactory<ThreadedPlugin,IrMCSyncConfig>();
	}
}

ThreadedPlugin::ThreadedPlugin( const KConfig *config )
		: Konnector( config ), mConfigWidget(0)
{
	kdDebug() << __PRETTY_FUNCTION__ << " this = " << this << endl;

	if ( config ) {
		mDeviceBtAddress = config->readEntry( "DeviceBtAddress" );
		mSyncCalendar =  config->readBoolEntry( "SyncCalendar" );
		mSyncAddressBook =  config->readBoolEntry( "SyncAddressBook" );
	}

	// Set the number of Threads;
	threadsNumber = 0;
	if(mSyncCalendar)
		++threadsNumber;
	if(mSyncAddressBook)
		++threadsNumber;

	// Set the Obex Client
	QObexBtTransport* btTransport = new QObexBtTransport;
	QString dest = mDeviceBtAddress;
	if ( !dest.isEmpty() )
		btTransport->setDestAddress( dest );

	btTransport->setUuid( QObexBtTransport::IrMCServ );

	QObexTransport* transport = btTransport;
	transport->setBlocking(true);
	bool verbose = true;
	obex = new Client( transport, verbose );

	obex->setUuid( QObexUuidIrMCSync );

	// We have to create the various clientManagers
	if(mSyncCalendar) {
		mCalendarClientManager = new ClientManager(CalendarSync, this);

		connect( mCalendarClientManager, SIGNAL(signalProgress(const KSync::Progress&)),
		         SLOT(slotProgress(const KSync::Progress&)) );
		connect( mCalendarClientManager, SIGNAL(signalError(const KSync::Error&)),
		         SLOT(slotError(const KSync::Error&)) );
		connect( mCalendarClientManager, SIGNAL(signalFinished()),
		         SLOT(slotFinished()) );
		connect( mCalendarClientManager, SIGNAL(signalSynceeReaded( KSync::Syncee* )),
		         SLOT(slotSynceeReaded( KSync::Syncee* )) );
		connect( mCalendarClientManager, SIGNAL(signalSynceeWrited()),
		         SLOT(slotSynceeWrited()) );

	}
	
	if(mSyncAddressBook) {
		mAddressBookClientManager = new ClientManager(AddressBookSync, this);

		connect( mAddressBookClientManager, SIGNAL(signalProgress(const KSync::Progress&)),
		         SLOT(slotProgress(const KSync::Progress&)) );
		connect( mAddressBookClientManager, SIGNAL(signalError(const KSync::Error&)),
		         SLOT(slotError(const KSync::Error&)) );
		connect( mAddressBookClientManager, SIGNAL(signalFinished()),
		         SLOT(slotFinished()) );
		connect( mAddressBookClientManager, SIGNAL(signalSynceeReaded( KSync::Syncee* )),
		         SLOT(slotSynceeReaded( KSync::Syncee* )) );
		connect( mAddressBookClientManager, SIGNAL(signalSynceeWrited()),
		         SLOT(slotSynceeWrited()) );

	}
}

ThreadedPlugin::~ThreadedPlugin()
{
	/*kdDebug() << __PRETTY_FUNCTION__ << " this = " << this << endl;
	mCalendarClientManager->terminate();
	mCalendarClientManager->wait();*/
}

SynceeList ThreadedPlugin::syncees()
{
	return mSyncees;
}

void ThreadedPlugin::writeConfig( KConfig *config )
{
	kdDebug() << "writeConfig" << endl;

	Konnector::writeConfig( config );

	config->writeEntry( "DeviceBtAddress", mDeviceBtAddress );
	config->writeEntry( "SyncCalendar", mSyncCalendar );
	config->writeEntry( "SyncAddressBook", mSyncAddressBook );
}

// They were removed on 2004-10-08, let's what happens now...

/* Kapabilities ThreadedPlugin::capabilities()
{
	Kapabilities caps;
	caps.setSupportMetaSyncing( true );
	caps.setSupportsPushSync( false );
	caps.setNeedsConnection( true );
	caps.setSupportsListDir( false );
	caps.setNeedsIPs( false );
	caps.setNeedsSrcIP( false );
	caps.setNeedsDestIP( false );
	caps.setAutoHandle( false );
	caps.setNeedAuthentication( false );

	QValueList<QPair<QString, QString> > user;
	user.append(qMakePair(QString::fromLatin1("root"), QString::fromLatin1("rootme") ) );
	caps.setUserProposals( user );

	QStringList ips;
	ips << "1.1.1.1";
	caps.setIpProposals( ips );

	// Model Stuff
	QStringList models;
	models << "IrMCSync Capable phone and other devices";
	caps.setModels( models );
	caps.setNeedsModelName( true );

	return caps;
}

void ThreadedPlugin::setCapabilities( const KSync::Kapabilities& )
{
	kdDebug() << __PRETTY_FUNCTION__ << " this = " << this << endl;
}*/

bool ThreadedPlugin::readSyncees()
{
	kdDebug() << __PRETTY_FUNCTION__ << " this = " << this << endl;


	readedSynceesNumber = 0;
	if(mSyncCalendar) {
		kdDebug() << " mSyncCalendar = yes" << endl;
		mCalendarClientManager->setObexClient(obex);
		return mCalendarClientManager->readSyncees();
	}
	else if(mSyncAddressBook) {
		kdDebug() << " mSyncAddressBook = yes" << endl;
		mAddressBookClientManager->setObexClient(obex);
		return mAddressBookClientManager->readSyncees();
	}
	return false;
}

bool ThreadedPlugin::writeSyncees()
{
	kdDebug() << __PRETTY_FUNCTION__ << " this = " << this << endl;
	if(mSyncCalendar) {
		kdDebug() << " mSyncCalendar = yes" << endl;
		mCalendarClientManager->setObexClient(obex);
		return mCalendarClientManager->writeSyncees();
	}
	else if(mSyncAddressBook) {
		kdDebug() << " mSyncAddressBook = yes" << endl;
		mAddressBookClientManager->setObexClient(obex);
		return mAddressBookClientManager->writeSyncees();
	}
	return false;
}

bool ThreadedPlugin::connectDevice()
{
	kdDebug() << "connectDevice()" << endl;

	return obex->connectClient();
}

bool ThreadedPlugin::disconnectDevice()
{
	kdDebug() << "disconnectDevice()" << endl;
	/* Free the mSyncees */
	mSyncees.clear();
	return obex->disconnectClient();
}

KSync::KonnectorInfo ThreadedPlugin::info() const
{
	kdDebug() << __PRETTY_FUNCTION__ << " this = " << this << endl;
	return KSync::KonnectorInfo();
}

void ThreadedPlugin::download( const QString& )
{
	kdDebug() << __PRETTY_FUNCTION__ << " this = " << this << endl;
}

void ThreadedPlugin::slotFinished()
{
	kdDebug() << __PRETTY_FUNCTION__ << " this = " << this << endl;
//	progress( KSync::StdProgress::done() );
}

// They were removed on 2004-10-08, let's what happens now... 
/*
void ThreadedPlugin::slotError( const KSync::Error& e )
{
	kdDebug() << __PRETTY_FUNCTION__ << " this = " << this << endl;
	error( e );
}

void ThreadedPlugin::slotProgress( const KSync::Progress& p )
{
	kdDebug() << __PRETTY_FUNCTION__ << " this = " << this << endl;
///	progress( p );
}*/

void ThreadedPlugin::slotSynceeReaded(KSync::Syncee* syncee)
{
	kdDebug() << "slotSynceeReceived()" << endl;

	mSyncees.append(syncee);

	if(++readedSynceesNumber == threadsNumber)
		emit synceesRead( this );
}

void ThreadedPlugin::slotSynceeWrited()
{
	kdDebug() << "slotSynceeWrited" << endl;

	emit synceesWritten( this );
}

#include "irmcsynckonnector.moc"
