/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004 by Michael Buesch                                  *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef __LISTVIEW_H
#define __LISTVIEW_H

#include <klistview.h>

#include <qcolor.h>


#define LIST_ENTRY_STATUS_VALID		(1)
#define LIST_ENTRY_STATUS_EXPIRED	(1 << 1)
#define LIST_ENTRY_STATUS_UPDATEINT	(1 << 2)

#define DEFAULT_COLOR_FOREGROUND	QColor(0, 0, 0)
#define DEFAULT_COLOR_BACKGROUND	QColor(255, 255, 255)

/** PwManager implementation of the list view.
  * Derived from KListView.
  */
class ListViewPwM : public KListView
{
	Q_OBJECT
public:
	ListViewPwM(QWidget *parent = 0, const char *name = 0);

	/** get the category name we are currently displaying */
	QString getCat() const
			{ return curCat; }
	/** set the category name */
	void setCat(const QString &cat)
			{ curCat = cat; }

signals:
	void layoutChanged();

protected:
	bool event(QEvent *e);

protected:
	/** category name we are displaying with this view */
	QString curCat;
};

class ListViewItemPwM : public QCheckListItem
{
public:
	ListViewItemPwM(QListView *parent,
			unsigned int revision,
			unsigned int _docPos);

	/** returns the revision counter */
	unsigned int getRev() const
			{ return rev; }
	/** set the revision counter */
	void setRev(unsigned int _rev)
			{ rev = _rev; }
	/** returns the doc pos of this entry */
	unsigned int getDocPos() const
			{ return docPos; }
	/** set the doc pos */
	void setDocPos(unsigned int _docPos)
			{ docPos = _docPos; }

public: /* status values */
	void setStatusValid(bool valid)
		{
			if (valid) {
				status |= LIST_ENTRY_STATUS_VALID;
			} else {
				status &= ~LIST_ENTRY_STATUS_VALID;
			}
		}
	bool getStatusValid() const
		{
			return (status & LIST_ENTRY_STATUS_VALID);
		}
	void setStatusExpired(bool expired)
		{
			if (expired) {
				status |= LIST_ENTRY_STATUS_EXPIRED;
			} else {
				status &= ~LIST_ENTRY_STATUS_EXPIRED;
			}
		}
	bool getStatusExpired() const
		{
			return (status & LIST_ENTRY_STATUS_EXPIRED);
		}
	void setStatusUpdateIntOver(bool update)
		{
			if (update) {
				status |= LIST_ENTRY_STATUS_UPDATEINT;
			} else {
				status &= ~LIST_ENTRY_STATUS_UPDATEINT;
			}
		}
	bool getStatusUpdateIntOver() const
		{
			return (status & LIST_ENTRY_STATUS_UPDATEINT);
		}

public:	/* set status colors */
	void setColorDefault()
		{
			background = DEFAULT_COLOR_BACKGROUND;
			foreground = DEFAULT_COLOR_FOREGROUND;
		}
	void setColorInvalid()
		{
			background.setRgb(255, 255, 255);
			foreground.setRgb(128, 128, 128);
		}
	void setColorExpired()
		{
			background.setRgb(255, 0, 0);
			foreground.setRgb(0, 0, 0);
		}
	void setColorUpdateIntOver()
		{
			background.setRgb(0, 0, 255);
			foreground.setRgb(0, 0, 0);
		}

protected:
	void paintCell(QPainter *p, const QColorGroup &_cg,
		       int column, int width, int align);

protected:
	/** pixmap to display for an item with state "on" */
	static QPixmap *onPix;
	/** pixmap to display for an item with state "off" */
	static QPixmap *offPix;
	/** item status */
	unsigned int status;
	/** revision counter number of this entry */
	unsigned int rev;
	/** position number in document for this entry */
	unsigned int docPos;
	/** color used for text painting */
	QColor foreground;
	/** color used for background painting */
	QColor background;
};

#endif
